import requests

from .endpoints import API_PATH, API_DATA, ROOT

class InvalidMethod(Exception):
    '''Invalid method was passed.'''

class RequestHandler:
    '''Handles all requests sent to the Sellix API'''

    def get(**kwargs):
        return requests.get(
            API_PATH[kwargs['endpoint']] + '/{}'.format(kwargs['uniqid']),
            headers=kwargs['authorization']
        ).json()

    def list(**kwargs):
        return requests.get(
            API_PATH[kwargs['endpoint']] + '?page={page}'.format(page=kwargs['page']) if 'page' in kwargs else API_PATH[kwargs['endpoint']],
            headers=kwargs['authorization']
        ).json()

    def post(**kwargs):
        API, DATA = API_PATH[kwargs['endpoint']] + '/{}'.format(kwargs['uniqid']) if 'uniqid' in kwargs else API_PATH[kwargs['endpoint']], API_DATA[kwargs['endpoint']] if kwargs['endpoint'] in API_DATA else None

        if DATA:
            for key, value in DATA.items():
                if value:
                    DATA[key] = kwargs[key]

                else:
                    if key in kwargs:
                        DATA[key] = kwargs[key]

                    else:
                        DATA[key] = None

        return requests.post(
            API,
            json=DATA,
            headers=kwargs['authorization']
        ).json()

    def edit(**kwargs):
        API, DATA = API_PATH[kwargs['endpoint']] + '/{}'.format(kwargs['uniqid']), API_DATA[kwargs['endpoint']]

        for key, value in DATA.items():
            if key in kwargs:
                DATA[key] = kwargs[key]

            else:
                DATA[key] = None

        return requests.put(
            API,
            json=DATA,
            headers=kwargs['authorization']
        ).json()

    def delete(**kwargs):
        API = API_PATH[kwargs['endpoint']] + '/{}'.format(kwargs['uniqid'])

        return requests.delete(
            API,
            headers=kwargs['authorization']
        ).json()

class Sellix:
    '''The main class that the user will interact with'''

    def __init__(self, secret_key):
        self.secret_key = secret_key

        self.authorization = {
            'Authorization': 'Bearer {secret_key}'.format(secret_key=secret_key)
        }
        self.handles = {
            'get': RequestHandler.get,
            'list': RequestHandler.list,
            'edit': RequestHandler.edit,
            'post': RequestHandler.post,
            'create': RequestHandler.post,
            'delete': RequestHandler.delete,
        }

    def check(self, method, endpoint, **kwrags):
        API = self.handles.get(method, 'InvalidMethod')
        DATA = API_DATA.get(endpoint)

        if not callable(API):
            raise KeyError('Method does not exist.')

        if DATA:
            for key, value in DATA.items():
                if value:
                    if key not in kwrags:
                        raise KeyError('Missing required element {} inside the request data for {}'.format(key, endpoint))

    def request(self, method, endpoint, **kwargs):
        self.check(method, endpoint, **kwargs)

        kwargs['endpoint'] = endpoint
        kwargs['authorization'] = self.authorization

        return self.handles[method](**kwargs)



'''List of all endpoints Stelix knows about.'''

ROOT = 'https://dev.sellix.io/v1'
API_PATH  = {
    'order'             : ROOT + '/orders',
    'coupon'            : ROOT + '/coupons',
    'products'          : ROOT + '/products',
    'payments'          : ROOT + '/payments',
    'feedback'          : ROOT + '/feedback',
    'blacklists'        : ROOT + '/blacklists',
    'categories'        : ROOT + '/categories',

    # Generic Endpoints

    'queries'           : ROOT + '/queries',
    'queries-reply'     : ROOT + '/queries/reply',
    'queries-close'     : ROOT + '/queries/close',
    'queries-reopen'    : ROOT + '/queries/reopen',

    # Specific Query Endpoints
}
API_DATA = {
    'categories': {
        'title': 1,
        'unlisted': 0,
        'sort_priority': 0,
        'products_bound': 0
    },
    'blacklists': {
        'type': 1,
        'data': 1,
        'note': 0
    },
    'feedback': {
        'reply': 1
    },
    'products': {
        'title': 1,
        'description': 1,
        'price': 1,
        'gateways': 1,
        'type': 1,
        'discount_value': 1,
        'currency': 0,
        'quantity': 0,
        'stock_delimiter': 0,
        'serials': 0,
        'serials_remove_duplicates': 0,
        'delivery_text': 0,
        'service_text': 0,
        'stock': 0,
        'custom_fields': 0,
        'crypto_confirmations_needed': 0,
        'max_risk_leve': 0,
        'block_vpn_proxies': 0,
        'sort_priority': 0,
        'unlisted': 0,
        'terms_of_service': 0,
        'warranty': 0,
        'warranty_text': 0,
        'private': 0,
        'webhooks': 0
    },
    'payments': {
        'cart': 0,
        'confirmations': 1,
        'coupon_code': 0,
        'credit_card': 0,
        'currency': 1,
        'custom_fields': 0,
        'email': 1,
        'fraud_shield': 0,
        'gateways': 1,
        'lex_payment_method': 0,
        'paypal_apm': 0,
        'product_addons': 0,
        'product_variations': 0,
        'quantity': 1,
        'title': 0,
        'value': 0,
        'webhook': 0,
        'white_label': 0
    },
    'coupon': {
        'code': 1,
        'discount_value': 1,
        'max_uses': 0,
        'products_bound': 0
    },
    'queries-reply': {
        'reply': 1
    }
}



import stelix

sellix = stelix.Sellix(
    secret_key='SECRET_KEY'
)

coupon = sellix.request(
    method='create', # What you want to do, in this case we want to create a new coupon code
    endpoint='coupon', # The API Endpoint you are trying to access

    code='stelix is awesome',
    discount_value=0.15,
    max_uses=5,
    products_bound=[
        'stelix2021abc'
    ]

    # Passing the arguments required to create the coupon code, some are
    # required while some are not, read more at https://developers.sellix.io
)

print(coupon['data']['uniqid']) # Getting the ID of the newly created coupon code



import stelix

sellix = stelix.Sellix(
    secret_key='SECRET_KEY'
)

payment = sellix.request(
    method='create', # What you want to do, in this case we want to create a new coupon code
    endpoint='payments', # The API Endpoint you are trying to access

    title='Developer Title',
    gateways=[
        'BITCOIN'
    ],
    email='email@gmail.com',
    currency='USD',

    value=26,
    quantity=1,

    confirmations=1,
    white_label=True
)

order = sellix.request(
    method='get',
    endpoint='order',

    uniqid=payment['data']['invoice']['uniqid']
)

print(order['data']['order']['status_history'][0]['status']) # A successfully completed order should have the "COMPLETED" tag
